/**
 * Copyright (c) 2008-2011 Sonatype, Inc.
 *
 * All rights reserved. Includes the third-party code listed at http://www.sonatype.com/products/nexus/attributions.
 * Sonatype and Sonatype Nexus are trademarks of Sonatype, Inc. Apache Maven is a trademark of the Apache Foundation.
 * M2Eclipse is a trademark of the Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
package org.sonatype.nexus.plugins.aether.rest;

import org.apache.maven.index.artifact.Gav;
import org.apache.maven.index.artifact.IllegalArtifactCoordinateException;
import org.codehaus.plexus.component.annotations.Component;
import org.codehaus.plexus.component.annotations.Requirement;
import org.restlet.Context;
import org.restlet.data.Request;
import org.restlet.data.Response;
import org.restlet.data.Status;
import org.restlet.resource.ResourceException;
import org.restlet.resource.Variant;
import org.sonatype.aether.graph.Dependency;
import org.sonatype.aether.graph.DependencyNode;
import org.sonatype.aether.util.listener.AbstractRepositoryListener;
import org.sonatype.nexus.plugins.aether.NexusAether;
import org.sonatype.nexus.plugins.aether.workspace.NexusWorkspace;
import org.sonatype.nexus.proxy.maven.MavenRepository;
import org.sonatype.nexus.proxy.registry.RepositoryRegistry;
import org.sonatype.nexus.rest.AbstractNexusPlexusResource;
import org.sonatype.plexus.rest.resource.PathProtectionDescriptor;
import org.sonatype.plexus.rest.resource.PlexusResource;

@Component( role = PlexusResource.class, hint = "SimpleAetherPlexusResource" )
public class SimpleAetherPlexusResource
    extends AbstractNexusPlexusResource
{

    private static final String GROUP_ID = "groupId";

    private static final String ARTIFACT_ID = "artifactId";

    private static final String VERSION = "version";

    @Requirement
    private NexusAether nexusAether;

    @Requirement
    private RepositoryRegistry repositoryRegistry;

    @Override
    public String getResourceUri()
    {
        return "/aether/{" + GROUP_ID + "}/{" + ARTIFACT_ID + "}/{" + VERSION + "}";
    }

    @Override
    public PathProtectionDescriptor getResourceProtection()
    {
        return null;
    }

    @Override
    public Object getPayloadInstance()
    {
        return null;
    }

    @Override
    public Object get( Context context, Request request, Response response, Variant variant )
        throws ResourceException
    {
        String groupId = request.getAttributes().get( GROUP_ID ).toString();
        String artifactId = request.getAttributes().get( ARTIFACT_ID ).toString();
        String version = request.getAttributes().get( VERSION ).toString();

        Dependency dependency;
        try
        {
            dependency = nexusAether.createDependencyFromGav( new Gav( groupId, artifactId, version ), "compile" );
        }
        catch ( IllegalArtifactCoordinateException e )
        {
            throw new ResourceException( Status.CLIENT_ERROR_BAD_REQUEST, e );
        }

        NexusWorkspace nexusWorkspace =
            nexusAether.createWorkspace( repositoryRegistry.getRepositoriesWithFacet( MavenRepository.class ) );

        DependencyNode root;
        try
        {
            root = nexusAether.collectDependencies( nexusWorkspace, dependency, false, false, new AbstractRepositoryListener()
            {
                    } );
        }
        catch ( Exception e )
        {
            throw new ResourceException( Status.SERVER_ERROR_INTERNAL, e );
        }

        return root;
    }
}
